-- Test invoice_attachment
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(29);

set search_path to numerus, auth, public;

select has_table('invoice_attachment');
select has_pk('invoice_attachment' );
select table_privs_are('invoice_attachment', 'guest', array []::text[]);
select table_privs_are('invoice_attachment', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('invoice_attachment', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('invoice_attachment', 'authenticator', array []::text[]);

select has_column('invoice_attachment', 'invoice_id');
select col_is_pk('invoice_attachment', 'invoice_id');
select col_is_fk('invoice_attachment', 'invoice_id');
select fk_ok('invoice_attachment', 'invoice_id', 'invoice', 'invoice_id');
select col_type_is('invoice_attachment', 'invoice_id', 'integer');
select col_not_null('invoice_attachment', 'invoice_id');
select col_hasnt_default('invoice_attachment', 'invoice_id');

select has_column('invoice_attachment', 'original_filename');
select col_type_is('invoice_attachment', 'original_filename', 'text');
select col_not_null('invoice_attachment', 'original_filename');
select col_hasnt_default('invoice_attachment', 'original_filename');

select has_column('invoice_attachment', 'mime_type');
select col_type_is('invoice_attachment', 'mime_type', 'text');
select col_not_null('invoice_attachment', 'mime_type');
select col_hasnt_default('invoice_attachment', 'mime_type');

select has_column('invoice_attachment', 'content');
select col_type_is('invoice_attachment', 'content', 'bytea');
select col_not_null('invoice_attachment', 'content');
select col_hasnt_default('invoice_attachment', 'content');


set client_min_messages to warning;
truncate invoice_attachment cascade;
truncate invoice cascade;
truncate tax cascade;
truncate tax_class cascade;
truncate contact_tax_details cascade;
truncate contact cascade;
truncate company_user cascade;
truncate payment_method cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into tax_class (tax_class_id, company_id, name)
values (22, 2, 'vat')
     , (44, 4, 'vat')
;

insert into tax (tax_id, company_id, tax_class_id, name, rate)
values (3, 2, 22, 'IVA 21 %', 0.21)
     , (6, 4, 44, 'IVA 10 %', 0.10)
;

insert into contact (contact_id, company_id, name)
values ( 9, 2, 'Customer 1')
     , (10, 4, 'Customer 2')
;

insert into contact_tax_details (contact_id, business_name, vatin, address, city, province, postal_code, country_code)
values ( 9, 'Customer 1 Ltd', 'XXX123', 'Fake St.', 'City', 'Province', '17480', 'ES')
     , (10, 'Customer 2 Ltd', 'XXX234', 'Fake St.', 'City', 'Province', '17480', 'ES')
;

insert into invoice (invoice_id, company_id, invoice_number, contact_id, invoice_date, payment_method_id, currency_code)
values (13, 2, 'INV001',  9, '2011-01-11', 222, 'EUR')
     , (14, 4, 'INV002', 10, '2022-02-22', 444, 'EUR')
;

insert into invoice_attachment (invoice_id, original_filename, mime_type, content)
values (13, 'invoice.txt', 'text/plain', convert_to('IOU 42', 'UTF8'))
     , (14, 'invoice.html', 'text/html', convert_to('<html>IOU <em>42</em></html>', 'UTF8'))
;

prepare invoice_attachment_data as
select invoice_id, original_filename
from invoice_attachment
order by invoice_id, original_filename;

set role invoicer;
select is_empty('invoice_attachment_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'invoice_attachment_data',
	$$ values (13, 'invoice.txt')
	$$,
	'Should only list tax of products of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'invoice_attachment_data',
	$$ values (14, 'invoice.html')
	$$,
	'Should only list tax of products of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'invoice_attachment_data',
	'42501', 'permission denied for table invoice_attachment',
	'Should not allow select to guest users'
);
reset role;


select *
from finish();

rollback;

